/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.tools.verifier.apiscan.classfile;

import java.lang.ref.SoftReference;
import java.util.HashSet;
import java.util.Set;

import com.sun.corba.ee.org.objectweb.asm.AnnotationVisitor;
import com.sun.corba.ee.org.objectweb.asm.commons.EmptyVisitor;

/**
 * This class represents the information about a field in a Java class after
 * parsing using ASM
 * 
 * @author Seema Richard
 */
public class ASMField extends EmptyVisitor implements Field {
	private Set<Annotation> annotations = new HashSet<Annotation>();
	private SoftReference<ClassFile> owningClass;
	private String descriptor;
	private int access;
	private String signature;
	private String name;
	private Object value;

	public ASMField(ClassFile owningClass, String name, String descriptor,
			int access, String signature, Object value) {
		this.owningClass = new SoftReference<ClassFile>(owningClass);
		this.name = name;
		this.descriptor = descriptor;
		this.access = access;
		this.signature = signature;
		this.value = value;
	}

	public Set<Annotation> getAnnotations() {
		return annotations;
	}

	public void setAnnotations(Set<Annotation> annotations) {
		this.annotations = annotations;
	}

	public SoftReference<ClassFile> getOwningClass() {
		return owningClass;
	}

	public void setOwningClass(SoftReference<ClassFile> owningClass) {
		this.owningClass = owningClass;
	}

	public String getDescriptor() {
		return descriptor;
	}

	public void setDescriptor(String descriptor) {
		this.descriptor = descriptor;
	}

	public int getAccess() {
		return access;
	}

	public void setAccess(int access) {
		this.access = access;
	}

	public String getSignature() {
		return signature;
	}

	public void setSignature(String signature) {
		this.signature = signature;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Object getValue() {
		return value;
	}

	public void setValue(Object value) {
		this.value = value;
	}

	@Override
	public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
		String name = Util.convertToExternalClassName(desc);
		String annName = name.substring(1, name.length() - 1);
		ASMAnnotation annotation = new ASMAnnotation();
		annotation.setName(annName);
		annotations.add(annotation);
		// annotation.setDescription(desc);
		return this;
	}

}
