/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.enterprise.tools.verifier.tests.appclient;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import com.sun.enterprise.deployment.ApplicationClientDescriptor;
import com.sun.enterprise.tools.verifier.Result;
import com.sun.enterprise.tools.verifier.SpecVersionMapper;
import com.sun.enterprise.tools.verifier.apiscan.classfile.ClassFileLoader;
import com.sun.enterprise.tools.verifier.tests.ComponentNameConstructor;
import com.sun.enterprise.tools.verifier.tests.util.AnnotationClassesLoadableHelper;

/**
 * Verifies if all the annotation classes used in the application client module
 * are loadable from the classpath.
 * 
 * @author Seema Richard
 */

public class AppClientAnnotationClassesLoadable extends AppClientTest implements
		AppClientCheck {

	public Result check(ApplicationClientDescriptor descriptor) {
		Result result = getInitializedResult();
		String archiveUri = getAbstractArchiveUri(descriptor);
		ComponentNameConstructor compName = getVerifierContext()
				.getComponentNameConstructor();
		Set<String> classNames = descriptor.getComponentClassNames();
		ClassFileLoader classFileLoader = getVerifierContext()
				.getClassFileLoader();

		boolean allPassed = true;
		String specVer = SpecVersionMapper
				.getAppClientVersion(getVerifierContext().getJavaEEVersion());
		String apiName = "appclient_" + specVer; // NOI18N
		
		List<String> failedMessages = new ArrayList<String>();
		for (String className : classNames) {
			allPassed = allPassed
					&& AnnotationClassesLoadableHelper
							.getAnnotationLoadabilityStatus(className,
									classFileLoader, apiName, failedMessages);

		}
		if (allPassed) {
			result.setStatus(Result.PASSED);
			addGoodDetails(result, compName);
			result
					.passed(smh
							.getLocalString(
									getClass().getName() + ".passed",
									"All the annotation classes are loadable within [ {0} ] without any linkage error.",
									new Object[] { archiveUri }));
		} else {
			result.setStatus(Result.FAILED);
			addErrorDetails(result, compName);
			for (String failedMessage : failedMessages) {
				result.addErrorDetails(failedMessage);
			}
			result
					.addErrorDetails(smh
							.getLocalString(
									"com.sun.enterprise.tools.verifier.tests.loadableError",
									"Please either bundle the above mentioned annotation classes in the application "
											+ "or use optional packaging support for them."));
		}

		return result;
	}
}
