/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.enterprise.tools.verifier.tests.util;

import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import com.sun.enterprise.tools.verifier.Result;
import com.sun.enterprise.tools.verifier.apiscan.classfile.Annotation;
import com.sun.enterprise.tools.verifier.apiscan.classfile.ClassFile;
import com.sun.enterprise.tools.verifier.apiscan.classfile.ClassFileLoader;
import com.sun.enterprise.tools.verifier.apiscan.classfile.Field;
import com.sun.enterprise.tools.verifier.apiscan.classfile.Method;
import com.sun.enterprise.tools.verifier.apiscan.stdapis.APIRepository;

/**
 * This class is a helper for all the tests checking for the presence of
 * non-standard annotations.
 * 
 * @author Seema Richard
 */
public class AnnotationClassesLoadableHelper {
	/**
	 * This method is used to get the result of checking if any non standard
	 * annotations are being used in the class whose name is passed
	 * 
	 * @param className
	 *            Name of the class to be checked
	 * @param classFileLoader
	 *            The ClassFileLoader implementation passed
	 * @param apiName
	 *            The API version to which the descriptor belongs
	 * @param result
	 *            The verifier test result object to which the warning messages
	 *            can be added
	 * @return a boolean that indicates if all the annotation classes were
	 *         loadable from the classpath or not
	 */
	public static boolean getAnnotationLoadabilityStatus(String className,
			ClassFileLoader classFileLoader, String apiName,
			List<String> failedMessages) {
		boolean allPassed = true;
		try {
			ClassFile classFile = classFileLoader.load(className);
			Set<Annotation> annotations = classFile.getAnnotations();
			Collection<? extends Method> methods = classFile.getMethods();
			for (Method method : methods) {
				annotations.addAll(method.getAnnotations());
			}

			Collection<? extends Field> fields = classFile.getFields();
			for (Field field : fields) {
				annotations.addAll(field.getAnnotations());
			}
			failedMessages
					.add("The following annotation classes could not be loaded [");
			for (Annotation annotation : annotations) {
				ClassFile c = null;
				try {
					// Class c = loadClass(result, annotation.getName());
					boolean isStandardAnnotation = APIRepository.Instance()
							.isClassPartOf(annotation.getName(), apiName);
					// System.out.println("The annotation class is standard "
					// + isStandardAnnotation + annotation.getName());
					if (isStandardAnnotation) {
						continue;
					}
					c = classFileLoader.load(annotation.getName());
				} catch (IOException e) {
					failedMessages.add(annotation.getName());
					allPassed = false;
				}
			}
			failedMessages.add("]");
		}

		catch (IOException e) {
			failedMessages.add(" Class could not be loaded" + className);
			allPassed = false;
		}
		return allPassed;
	}
}
